/******************************************************************************
 * Bergen Sound Server Spatialized Edition                                    *
 * SpatializedBergenTest Demo Application Sourcefile                          *
 *                                                                            *
 * Copyright 2002 Severin S. Todt - University of Applied Sciences Wedel      *
 *                s.todt@fh-wedel.de                                          *
 *                                                                            *
 * The Bergen Sound Server in current release is based on the                 *
 * Bergen Sound Server, edited 2000 by Dave Pape                              *
 *                                                                            *
 * The Bergen Sound Server is developed within a project at the               *
 * Electronic Visualization Laboratory at the University of Illinois at       *
 * Chicago (www.evl.uic.edu) and is part of the master thesis written by      *
 * Severin S. Todt at the University of Applied Sciences - Wedel.             *
 * (www.fh-wedel.de)                                                          *
 *                                                                            *
 ******************************************************************************
 * Modification history :                                                     *
 * - initial setup                                                            *
 *                                                                            *
 *                                                   last modified : 09/20/02 *
 ******************************************************************************/
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <math.h>
#include <GL/glut.h>
#include "bergenServer.h"
#include "bergenListener.h"
#include "bergenSample.h"


enum SOUNDTYPE {TONE,WHITENOISE,SAMPLE};
enum MOVESCHEME {NO_MOVESCHEME,DISTANCE_RIGHT,DISTANCE_FRONT,DISTANCE_LEFT,DISTANCE_BACK,CIRCLE_HORIZONTAL,CIRCLE_VERTICAL,SPIRAL};

float listenerX=0.0, listenerZ=0.0, sourceX=0.0, sourceZ=0.0;
int listenerMovement=0, sourceMovement=1;
bergenServer * server = new bergenServer();
bergenSound * sound;

void keyboard(unsigned char key, int x, int y){
switch(key){
	case 'l' :
		if(listenerMovement)
			listenerX-=0.01;
		else
			sourceX-=0.01;
		break;
	case 'r' :
		if(listenerMovement)
			listenerX+=0.01;
		else
			sourceX+=0.01;
		break;
	case 'u' :
		if(listenerMovement)
			listenerZ-=0.01;
		else
			sourceZ-=0.01;
		break;
	case 'd' :
		if(listenerMovement)
			listenerZ+=0.01;
		else
			sourceZ+=0.01;
		break;
	case 'L':
		sourceMovement=0;
		listenerMovement=1;
		break;
	case 'S':
		sourceMovement=1;
		listenerMovement=0;
		break;
	case 'Q':
		sound->stop();
		delete(sound);
		delete(server);
		break;
	default :
		break;
	}
glutPostRedisplay();
}

void reshape(int w, int h){
glViewport(0,0,(GLsizei)w,(GLsizei)h);
glMatrixMode(GL_PROJECTION);
glLoadIdentity();
glOrtho(-2.0,2.0,-2.0,2.0,1.5,20.0);
glMatrixMode(GL_MODELVIEW);
glutPostRedisplay();
}

void display(void){
glClear(GL_COLOR_BUFFER_BIT);
glLoadIdentity();
gluLookAt(0.0,5.0,0.0,0.0,0.0,0.0,0.0,0.0,-1.0);
glBegin(GL_LINES);
	glVertex3f(-1.0,0.0,-1.0);
	glVertex3f(1.0,0.0,-1.0);

	glVertex3f(1.0,0.0,-1.0);
	glVertex3f(1.0,0.0,1.0);

	glVertex3f(1.0,0.0,1.0);
	glVertex3f(-1.0,0.0,1.0);

	glVertex3f(-1.0,0.0,1.0);
	glVertex3f(-1.0,0.0,-1.0);
glEnd();
glPushMatrix();
	server->listener()->setPosition(listenerX,0.0,listenerZ);
	glColor3f(0.0,1.0,0.0);
	glTranslatef(listenerX,0.0,listenerZ);
	glutSolidSphere(0.2,15,10);
glPopMatrix();
glPushMatrix();
	sound->setPosition(sourceX,0.0,sourceZ);
	glColor3f(1.0,0.0,0.0);
	glTranslatef(sourceX,0.0,sourceZ);
	glutSolidSphere(0.1,15,10);
glPopMatrix();
glutSwapBuffers();
}

void init(void){
glClearColor(0.0,0.0,0.0,0.0);
glColor3f(1.0,1.0,1.0);
}

void initGlut(int argc, char **argv){
glutInit(&argc,argv);
glutInitDisplayMode(GLUT_RGB|GLUT_DOUBLE);
glutInitWindowSize(500,500);
glutInitWindowPosition(0,0);
glutCreateWindow("Spatialized Sound OpenGL Demo");
glutDisplayFunc(display);
glutReshapeFunc(reshape);
glutKeyboardFunc(keyboard);

sound->play();

glutMainLoop();
}

int main(int argc,char **argv){
if (argc < 2)
    {
    fprintf(stderr, "usage: %s infile\n", argv[0]);
    fprintf(stderr, "\n");
    fprintf(stderr, "S - toggle to move sound\n");
    fprintf(stderr, "L - toggle to move listener\n");
    fprintf(stderr, "l - move left\n");
    fprintf(stderr, "r - move right\n");
    fprintf(stderr, "u - move up\n");
    fprintf(stderr, "d - move down\n");
    fprintf(stderr, "Q - quit\n");
    exit(1);
    }
sound = new bergenSample(argv[1],server);
((bergenSample*)sound)->setLoop(1);
sound->setAttenuationType(INVERSE_SQUARE_LAW);

server->listener()->setPosition(0.0,0.0,0.0);

init();

initGlut(argc,argv);

return 0;
}
